'use strict';

var Sortable = require('sortablejs');
var Cascader = require('cascaderjs');
var i18nJs = require('i18n-js');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var Sortable__default = /*#__PURE__*/_interopDefault(Sortable);
var Cascader__default = /*#__PURE__*/_interopDefault(Cascader);

// src/lib.ts

// src/types.ts
function isLispyOperator(value) {
  return typeof value === "string" && ["not", "and", "or"].includes(value);
}
function isLispyConditionExpr(expr) {
  if (!Array.isArray(expr)) {
    return false;
  }
  if (expr.length !== 3) {
    return false;
  }
  if (expr[0] !== "=") {
    return false;
  }
  if (typeof expr[1] !== "string") {
    return false;
  }
  const thirdElement = expr[2];
  const isValidThirdElement = typeof thirdElement === "string" || typeof thirdElement === "number" || typeof thirdElement === "boolean" || Array.isArray(thirdElement);
  return isValidThirdElement;
}
function isLispyNotExpr(expr) {
  return Array.isArray(expr) && expr.length === 2 && expr[0] === "not" && isLispyExpression(expr[1]);
}
function isLispyAndOrExpr(expr) {
  if (!Array.isArray(expr) || expr.length < 2) {
    return false;
  }
  const [operator, ...operands] = expr;
  return (operator === "and" || operator === "or") && operands.every((operand) => isLispyExpression(operand));
}
function isLispyExpression(expr) {
  return isLispyNotExpr(expr) || isLispyAndOrExpr(expr) || isLispyConditionExpr(expr);
}
function isSchemaField(obj) {
  return typeof obj === "object" && obj !== null && typeof obj.verbose_name === "string" && typeof obj.class === "string" && (obj.choices === null || Object.prototype.toString.call(obj.choices) === "[object Object]" && Object.values(obj.choices).every((v) => typeof v === "string"));
}

// trans/zh-CN.json
var zh_CN_default = {
  operator: {
    not: "\u5426",
    and: "\u540C\u65F6\u6EE1\u8DB3",
    or: "\u5176\u4E00\u6EE1\u8DB3"
  },
  "operator-tip": {
    not: "\u5BF9\u6761\u4EF6\u7684\u5426\u5B9A/\u53D6\u53CD",
    and: "\u540C\u65F6\u6EE1\u8DB3\u62EC\u53F7\u5185\u7684\u6240\u6709\u6761\u4EF6",
    or: "\u6EE1\u8DB3\u62EC\u53F7\u5185\u7684\u6761\u4EF6\u4E4B\u4E00"
  },
  delete: "\u5220\u9664",
  "toggle-not": "\u5207\u6362\u201C\u5426\u201D\u7B26\u53F7",
  fieldSelector: {
    placeholder: "\u8BF7\u9009\u62E9\u4E00\u4E2A\u5B57\u6BB5"
  },
  lookup: {
    exact: "\u7B49\u4E8E",
    iexact: "\u7B49\u4E8E\uFF08\u4E0D\u533A\u5206\u5927\u5C0F\u5199\uFF09",
    isnull: "\u4E3A\u7A7A",
    gt: "\u5927\u4E8E",
    gte: "\u5927\u4E8E\u6216\u7B49\u4E8E",
    lt: "\u5C0F\u4E8E",
    lte: "\u5C0F\u4E8E\u6216\u7B49\u4E8E",
    contains: "\u5305\u542B",
    icontains: "\u5305\u542B\uFF08\u4E0D\u533A\u5206\u5927\u5C0F\u5199\uFF09",
    startswith: "\u4EE5...\u5F00\u59CB",
    istartswith: "\u4EE5...\u5F00\u59CB\uFF08\u4E0D\u533A\u5206\u5927\u5C0F\u5199\uFF09",
    endswith: "\u4EE5...\u7ED3\u675F",
    iendswith: "\u4EE5...\u7ED3\u675F\uFF08\u4E0D\u533A\u5206\u5927\u5C0F\u5199\uFF09",
    regex: "\u6B63\u5219\u8868\u8FBE\u5F0F\u5339\u914D",
    iregex: "\u6B63\u5219\u8868\u8FBE\u5F0F\u5339\u914D\uFF08\u4E0D\u533A\u5206\u5927\u5C0F\u5199\uFF09"
  },
  "add-new": "\u521B\u5EFA\u4E00\u4E2A\u65B0\u7684\u6761\u4EF6\u6216\u6761\u4EF6\u96C6\u5408",
  "add-new-and-group": "\u521B\u5EFA\u201C\u540C\u65F6\u6EE1\u8DB3\u201D\u6761\u4EF6\u96C6\u5408",
  "add-new-or-group": "\u521B\u5EFA\u201C\u5176\u4E00\u6EE1\u8DB3\u201D\u6761\u4EF6\u96C6\u5408",
  "add-new-condition": "\u521B\u5EFA\u4E00\u4E2A\u65B0\u7684\u6761\u4EF6",
  "button-symbol": {
    N: "\u5426"
  }
};

// src/trans.ts
var i18n = new i18nJs.I18n({
  "en": {
    // for all en (en-US, etc.)
    "operator": {
      "not": "Not",
      "and": "And",
      "or": "Or"
    },
    "operator-tip": {
      "not": "Negation of the following condition",
      "and": "Must meet all conditions in parentheses",
      "or": "Must meet any one condition in parantheses"
    },
    "delete": "Delete",
    "toggle-not": "Toggle not operator",
    "fieldSelector": {
      "placeholder": "Select a field"
    },
    "lookup": {
      "exact": "equals",
      "iexact": "equals (case-insensitive)",
      "isnull": "is null",
      "gt": "is greater than",
      "gte": "is greater than or equal to",
      "lt": "is less than",
      "lte": "is less than or equal to",
      "contains": "contains",
      "icontains": "contains(case-insensitive)",
      "startswith": "starts with",
      "istartswith": "starts with(case-insensitive)",
      "endswith": "ends with",
      "iendswith": "ends with(case-insensitive)",
      "regex": "matches(regular expression)",
      "iregex": "matches(regular expression, case-insensitive)"
    },
    "add-new": "Add a new group or condition",
    "add-new-and-group": "Add a new `And` group",
    "add-new-or-group": "Add a new `Or` group",
    "add-new-condition": "Add a new condition",
    "button-symbol": {
      "N": "N"
    }
  },
  "zh": {
    // for all zh (zh-CN, etc.)
    ...zh_CN_default
  }
});
i18n.defaultLocale = "en";
i18n.enableFallback = true;
function trans(...params) {
  i18n.locale = navigator.language;
  return i18n.t(...params);
}

// src/components/operator-selector.ts
var OperatorSelect = class {
  element;
  isEditing = false;
  selectElement = null;
  options = [
    { value: "and", display: "And" },
    { value: "or", display: "Or" }
  ];
  constructor(element) {
    this.element = element;
    this.toggleEdit();
  }
  createSelect() {
    const select = document.createElement("div");
    select.className = "dlf:absolute dlf:bg-white dlf:border dlf:border-gray-300 dlf:rounded dlf:shadow-lg dlf:z-50";
    this.options.forEach((option) => {
      const optionElement = document.createElement("div");
      optionElement.className = `dlf-operator dlf-${option.value}-operator
          dlf:cursor-pointer dlf:p-2 dlf:hover:bg-gray-100`;
      optionElement.textContent = option.display;
      optionElement.setAttribute("data-value", option.value);
      optionElement.addEventListener("click", (e) => {
        e.stopPropagation();
        this.selectOption(option);
      });
      select.appendChild(optionElement);
    });
    const rect = this.element.getBoundingClientRect();
    select.style.top = `${rect.bottom}px`;
    select.style.left = `${rect.left}px`;
    return select;
  }
  toggleEdit() {
    if (!this.isEditing) {
      this.isEditing = true;
      this.selectElement = this.createSelect();
      document.body.appendChild(this.selectElement);
      const closeHandler = (e) => {
        if (this.selectElement && !this.selectElement.contains(e.target)) {
          this.closeSelect();
          document.removeEventListener("click", closeHandler);
        }
      };
      setTimeout(() => {
        document.addEventListener("click", closeHandler);
      }, 0);
    }
  }
  closeSelect() {
    if (this.selectElement) {
      this.selectElement.remove();
      this.selectElement = null;
    }
    this.isEditing = false;
  }
  selectOption(option) {
    this.element.textContent = option.display;
    this.element.className = `dlf-operator dlf-${option.value}-operator dlf:tooltip`;
    this.element.setAttribute("data-value", option.value);
    this.element.setAttribute("data-tip", option.display);
    this.closeSelect();
  }
};

// src/schema.ts
function genFieldsListFromSchema(curModelName, models, modelsInChain = /* @__PURE__ */ new Set()) {
  const res = [];
  const curModel = models[curModelName];
  if (!curModel) return res;
  modelsInChain.add(curModelName);
  const relatedModels = curModel["__rel"];
  for (const [fieldName, modelInfo] of Object.entries(relatedModels)) {
    const modelName = modelInfo.model;
    if (!modelsInChain.has(modelName)) {
      const model = models[modelName];
      if (model) {
        res.push({
          "value": fieldName,
          "label": modelInfo.verbose_name ?? model["__verbose_name"],
          "children": genFieldsListFromSchema(
            modelName,
            models,
            new Set(modelsInChain).add(modelName)
          )
        });
      }
    }
  }
  for (const [field, properties] of Object.entries(curModel)) {
    if (field === "__rel" || field === "__verbose_name") continue;
    if (!isSchemaField(properties)) {
      throw new Error(`Wrong schema! ${properties} should be of SchemaField type!`);
    }
    res.push({
      "value": field,
      "label": properties["verbose_name"]
    });
  }
  return res;
}
function getField(fieldNames, mainModel, modelsSchema) {
  if (fieldNames.length === 0) {
    throw new Error("fieldNames array must not be empty");
  }
  let curModel = modelsSchema[mainModel];
  for (let index = 0; index < fieldNames.length; index++) {
    const fieldName = fieldNames[index];
    if (index !== fieldNames.length - 1) {
      if (curModel[fieldName]) {
        throw new Error(`Field '${fieldName}' should not be a SchemaField in model`);
      }
      const nextModel = modelsSchema[curModel["__rel"][fieldName].model];
      if (!nextModel) {
        throw new Error(`Model '${fieldName}' not found in schema`);
      }
      curModel = nextModel;
    } else {
      const field = curModel[fieldName];
      if (!isSchemaField(field)) {
        throw new Error(`Wrong schema! '${fieldName}' should be a SchemaField`);
      }
      return field;
    }
  }
  throw new Error("Unexpected end of function execution");
}

// src/lib.ts
var NON_INITIALIZED_FIELD_SELECTER_QUERY = ".dlf-field-selector:not(:has(> .cascader-container))";
var CASCADER_SPLIT_STR = " > ";
function getValueInput(lookup, field, initialValue) {
  if (field.choices) {
    const selectElem = document.createElement("select");
    selectElem.className = "dlf-select";
    for (const [key, value] of Object.entries(field.choices)) {
      const optionElem = document.createElement("option");
      optionElem.value = key;
      optionElem.label = value;
      selectElem.append(optionElem);
    }
    if (initialValue) {
      if (initialValue in field.choices) {
        selectElem.value = initialValue;
      }
    }
    return selectElem;
  }
  const valueInputElem = document.createElement("input");
  valueInputElem.className = "dlf-input";
  if (initialValue) {
    valueInputElem.value = initialValue;
  }
  const field_class = field.class;
  let input_type = "";
  switch (field_class) {
    case "AutoField":
    case "BigAutoField":
    case "BigIntegerField":
    case "SmallAutoField":
    case "SmallIntegerField":
    case "DecimalField":
    case "DurationField":
    case "FloatField":
    case "IntegerField":
    case "PositiveBigIntegerField":
    case "PositiveIntegerField":
    case "PositiveSmallIntegerField":
    case "URLField":
    case "UUIDField":
      input_type = "number";
      break;
    case "BooleanField":
      input_type = "checkbox";
      break;
    case "CharField":
    case "EmailField":
    case "FileField":
    case "FilePathField":
    case "GenericIPAddressField":
    case "ImageField":
    case "SlugField":
    case "TextField":
      input_type = "text";
      break;
    case "DateField":
      input_type = "date";
      break;
    case "DateTimeField":
      input_type = "datetime-local";
      break;
    case "TimeField":
      input_type = "time";
      break;
  }
  if (lookup == "isnull") {
    input_type = "checkbox";
  }
  valueInputElem.type = input_type;
  return valueInputElem;
}
var FilterEditor = class {
  container;
  expression;
  schema;
  mainModel;
  fieldsList;
  getValueInputFn;
  constructor(options) {
    const container = typeof options.container === "string" ? document.querySelector(options.container) : options.container;
    if (!container)
      throw Error(`Cannot find element ${options.container}`);
    this.container = container;
    this.expression = options.initialExpression || ["and"];
    this.schema = options.schema;
    this.mainModel = options.mainModel;
    this.fieldsList = genFieldsListFromSchema(this.mainModel, this.schema.models);
    this.getValueInputFn = options.getValueInputFn ?? getValueInput;
    this.init();
  }
  init() {
    this.render(this.expression);
    this.setupEventListener();
    while (this.setupNewCascaderSelect()) {
    }
    this.setupSortableForGroup(this.container.querySelector(".dlf-group-body-conditions"));
  }
  setupSortableForGroup(containerElem) {
    Sortable__default.default.create(containerElem, {
      group: "dlf-sortable",
      // allow sort for nested elements
      filter: ".dlf-sortable-ignore",
      swapThreshold: 0.65,
      draggable: ".dlf-sortable",
      ghostClass: "dlf-sortable-ghost",
      chosenClass: "",
      dragClass: ""
    });
  }
  /**
   * setup a new CascaderSelect
   */
  setupNewCascaderSelect() {
    const fieldSelectorDivElem = document.querySelector(NON_INITIALIZED_FIELD_SELECTER_QUERY);
    if (!fieldSelectorDivElem) return null;
    const rawInitialValue = fieldSelectorDivElem.getAttribute("data-initial-value");
    let initialValue;
    if (rawInitialValue) {
      initialValue = rawInitialValue.split(",");
    }
    let prevCascaderValue;
    const cascader = new Cascader__default.default(
      NON_INITIALIZED_FIELD_SELECTER_QUERY,
      {
        mode: "single",
        placeholder: trans(["fieldSelector", "placeholder"]),
        data: this.fieldsList,
        showClear: false,
        defaultValue: initialValue,
        onChange: (value, _labelValue, _indexValue) => {
          if (value.length && prevCascaderValue != value) {
            const field = getField(value, this.mainModel, this.schema.models);
            this.setupLookupSelector(fieldSelectorDivElem.parentElement, field);
          }
          prevCascaderValue = value;
          fieldSelectorDivElem.setAttribute("data-value", value.toString());
        },
        displayRender(value) {
          return value.join(CASCADER_SPLIT_STR);
        }
      }
    );
    cascader.init();
    if (initialValue) {
      const field = getField(initialValue, this.mainModel, this.schema.models);
      this.setupLookupSelector(fieldSelectorDivElem.parentElement, field);
      fieldSelectorDivElem.removeAttribute("data-initial-value");
      fieldSelectorDivElem.setAttribute("data-value", initialValue.toString());
    }
    return cascader;
  }
  setupLookupSelector(conditionInputContainerElem, field) {
    const lookups = this.schema.lookups[field.class];
    const lookupSelectorElem = conditionInputContainerElem.querySelector(".dlf-lookup-selector");
    const valueInputDivElem = conditionInputContainerElem.querySelector(".dlf-value-input");
    const newLookupSelectorElem = document.createElement("select");
    newLookupSelectorElem.className = "dlf-select";
    lookups.forEach((lookup) => {
      const optionElem = document.createElement("option");
      optionElem.value = lookup;
      optionElem.label = trans(["lookup", lookup]);
      newLookupSelectorElem.append(optionElem);
    });
    const initial_value = lookupSelectorElem.getAttribute("data-initial-value");
    if (initial_value) {
      if (lookups.includes(initial_value)) {
        newLookupSelectorElem.value = initial_value;
      } else {
        console.error(`The initial_value ${initial_value} is not in lookups! Element:`, lookupSelectorElem);
      }
      lookupSelectorElem.removeAttribute("data-initial-value");
    }
    lookupSelectorElem.replaceChildren(newLookupSelectorElem);
    valueInputDivElem.replaceChildren();
    this.setupValueInput(conditionInputContainerElem, field, newLookupSelectorElem.value);
    newLookupSelectorElem.addEventListener("change", () => {
      this.setupValueInput(conditionInputContainerElem, field, newLookupSelectorElem.value);
    });
  }
  setupValueInput(conditionInputContainerElem, field, lookup) {
    const valueInputDivElem = conditionInputContainerElem.querySelector(".dlf-value-input");
    const initialValue = valueInputDivElem.getAttribute("data-initial-value");
    const valueInputElem = this.getValueInputFn(lookup, field, initialValue);
    valueInputDivElem.removeAttribute("data-initial-value");
    valueInputDivElem.replaceChildren(valueInputElem);
  }
  render(expression) {
    this.container.innerHTML = `
    <div class="dlf-root-container">
      ${this.renderExpression(expression)}
    </div>
    `;
  }
  renderExpression(expr) {
    let isNegated = false;
    let [operator, ...subExpressions] = expr;
    if (operator === "not") {
      if (subExpressions.length !== 1) {
        throw new Error("NOT expression must have exactly one subexpression");
      }
      isNegated = true;
      const nestedExpr = subExpressions[0];
      [operator, ...subExpressions] = nestedExpr;
    }
    if (operator == "=") {
      const conditionExpr = [operator, ...subExpressions];
      if (!isLispyConditionExpr(conditionExpr)) {
        throw Error(`${conditionExpr} is a valid LispyConditionExpr!`);
      }
      return this.renderCondition(true, conditionExpr);
    }
    const children = subExpressions.map((expr2) => {
      if (!isLispyExpression(expr2)) {
        throw new Error(`${expr2} is not a valid LispyExpression`);
      }
      return expr2[0] == "=" ? this.renderCondition(false, expr2) : this.renderExpression(expr2);
    }).join("");
    return `
    <div class="dlf-group dlf-sortable dlf-expanded dlf:group/group">
      <div class="dlf-group-prefix">
        ${isNegated ? this.renderOperator("not") : ""}
        ${this.renderOperator(operator)}
        <div class="dlf-group-collapse">
          <span class="dlf-parenthesis">(</span>
          <span class="dlf-ellipsis">...</span>
          <span class="dlf-parenthesis">)</span>
        </div>          
        <div class="dlf-group-hover-visible dlf:gap-2 dlf:flex dlf:items-center">
          ${this.renderCommonActions()}
        </div>
      </div>
        
      <div class="dlf-group-body">
        <div class="dlf-group-body-conditions">
          ${children}
        </div>
        <div class="dlf-group-body-actions dlf:relative dlf:px-2 dlf-sortable-ignore">
          <div title="${trans("add-new")}" >
            <button class="dlf-icon-button" data-action="add-new">
              <svg xmlns="http://www.w3.org/2000/svg" fill="none"
                viewBox="0 0 24 24" stroke-width="2.5" stroke="currentColor"
                class="dlf:size-4">
                <path stroke-linecap="round" stroke-linejoin="round" d="M12 4.5v15m7.5-7.5h-15" />
              </svg>
            </button>
          </div>
        </div>
      </div>
      <div class="dlf-parenthesis">)</div>
    </div>
    `;
  }
  renderOperator(operator) {
    let tip = trans(["operator-tip", operator]);
    return `
    <span class="dlf-operator dlf-${operator}-operator" title="${tip}"
      data-value="${operator}">
      ${trans(["operator", operator])}
    </span>
    `;
  }
  renderCondition(isNegated = false, condition) {
    let conditionInputContainerElem;
    if (condition) {
      const fields_lookup = condition[1].split("__");
      const fields = fields_lookup.slice(0, -1);
      const lookup = fields_lookup.slice(-1)[0];
      const value = condition[2];
      if (typeof value === "object") {
        throw new Error(`Currently doesn't support ${value} in LispyConditionExpr`);
      }
      conditionInputContainerElem = `
<div class="dlf-condition-input-container dlf:flex dlf:gap-2 dlf:items-center" >
  <div class="dlf-field-selector" data-initial-value="${fields}"></div>
  <div class="dlf-lookup-selector" data-initial-value="${lookup}"></div>
  <div class="dlf-value-input" data-initial-value="${value}"></div>
</div>
`;
    } else {
      conditionInputContainerElem = `
<div class="dlf-condition-input-container dlf:flex dlf:gap-2 dlf:items-center" >
  <div class="dlf-field-selector"></div>
  <div class="dlf-lookup-selector"></div>
  <div class="dlf-value-input"></div>
</div>
`;
    }
    return `
    <div class="dlf-condition dlf:group/condition dlf-sortable">
      ${isNegated ? this.renderOperator("not") : ""}
      ${conditionInputContainerElem}
      
      <div class="dlf:invisible dlf:group-hover/condition:visible dlf:gap-2 dlf:flex dlf:items-center">
        ${this.renderCommonActions()}
      </div>
    </div>
    `;
  }
  renderCommonActions() {
    return `
<div title="${trans("delete")}">
  <button class="dlf-icon-button" data-action="delete">
    <svg xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24"
      stroke-width="2.5" stroke="currentColor"
      class="dlf:size-4">
        <path stroke-linecap="round" stroke-linejoin="round" d="M6 18 18 6M6 6l12 12" />
    </svg>
  </button>
</div>

<div title="${trans("toggle-not")}" >
  <button class="dlf-icon-button"
    data-action="toggle-not">${trans(["button-symbol", "N"])}
  </button>
</div>
`;
  }
  setupEventListener() {
    this.container.addEventListener("click", (e) => {
      const target = e.target;
      if (target.closest(".dlf-group-collapse")) {
        const group = target.closest(".dlf-group");
        if (group) {
          group.classList.toggle("dlf-expanded");
        }
        return;
      }
      const btnElem = target.closest("button");
      if (btnElem) {
        const action = btnElem.dataset.action;
        switch (action) {
          case "add-new":
            this.actionAddNew(btnElem);
            break;
          case "delete":
            this.actionDelete(btnElem);
            break;
          case "toggle-not":
            this.actionToggleNot(btnElem);
            break;
        }
        return;
      }
      const operatorElem = target.closest(".dlf-operator");
      if (operatorElem && !operatorElem.classList.contains("dlf-not-operator")) {
        new OperatorSelect(operatorElem);
      }
    });
  }
  actionDelete(btnElem) {
    let parentElem = btnElem.closest(".dlf-condition");
    if (!parentElem) {
      parentElem = btnElem.closest(".dlf-group");
    }
    parentElem.remove();
    if (!this.container.firstElementChild.children.length) {
      this.render(["and"]);
      this.setupSortableForGroup(this.container.querySelector(".dlf-group-body-conditions"));
    }
  }
  actionAddNew(addNewBtnElem) {
    const actionContainerElem = addNewBtnElem.parentElement.parentElement;
    if (actionContainerElem.querySelector(".dlf-popup-menu")) return;
    const popupMenuElem = document.createElement("div");
    popupMenuElem.className = "dlf-popup-menu";
    popupMenuElem.innerHTML = `
<ul>
  <li><a data-action="add-new-and-group">${trans("add-new-and-group")}</a></li>
  <li><a data-action="add-new-or-group">${trans("add-new-or-group")}</a></li>
  <li><a data-action="add-new-condition">${trans("add-new-condition")}</a></li>
</ul>
`;
    const controller = new AbortController();
    const handleDocumentClick = (e) => {
      const target = e.target;
      if (!popupMenuElem.isConnected) return;
      const isClickInside = popupMenuElem.contains(target);
      const isAddButtonClick = addNewBtnElem.contains(target);
      if (!isClickInside && !isAddButtonClick) {
        popupMenuElem.remove();
        controller.abort();
      }
    };
    document.addEventListener("click", handleDocumentClick, {
      signal: controller.signal
    });
    popupMenuElem.querySelectorAll("a").forEach((menuItem) => {
      menuItem.addEventListener("click", () => {
        const action = menuItem.getAttribute("data-action");
        const groupBodyConditionsElem = actionContainerElem.previousElementSibling;
        switch (action) {
          case "add-new-and-group":
            this.addNewGroup("and", groupBodyConditionsElem);
            break;
          case "add-new-or-group":
            this.addNewGroup("or", groupBodyConditionsElem);
            break;
          case "add-new-condition":
            this.addNewCondition(groupBodyConditionsElem);
            break;
        }
        popupMenuElem.remove();
        controller.abort();
      }, { signal: controller.signal });
    });
    addNewBtnElem.parentElement.insertAdjacentElement("afterend", popupMenuElem);
  }
  addNewGroup(groupType, groupBodyConditionsElem) {
    const newGroupExpression = [groupType];
    const newGroupRawHTML = this.renderExpression(newGroupExpression);
    groupBodyConditionsElem.insertAdjacentHTML("beforeend", newGroupRawHTML);
    const newGroupElem = groupBodyConditionsElem.lastElementChild;
    this.setupSortableForGroup(newGroupElem.querySelector(".dlf-group-body-conditions"));
  }
  addNewCondition(groupBodyConditionsElem) {
    const newConditionHTML = this.renderCondition();
    groupBodyConditionsElem.insertAdjacentHTML("beforeend", newConditionHTML);
    this.setupNewCascaderSelect();
  }
  actionToggleNot(btnElem) {
    const closestConditionElem = btnElem.closest(".dlf-condition");
    let parentElem = null;
    if (closestConditionElem) {
      parentElem = closestConditionElem;
    } else {
      parentElem = btnElem.closest(".dlf-group-prefix");
    }
    if (!parentElem) {
      throw new Error("While executing 'toggle-not' action, parent element canot be found!");
    }
    const notOpElem = parentElem.querySelector(".dlf-not-operator");
    if (notOpElem) {
      notOpElem.remove();
    } else {
      parentElem.insertAdjacentHTML("afterbegin", this.renderOperator("not"));
    }
  }
  toJson() {
    const rootGroup = this.container.querySelector(".dlf-group");
    if (!rootGroup) {
      return ["and"];
    }
    return this.parseGroup(rootGroup);
  }
  parseGroup(groupElem) {
    const operatorElems = groupElem.firstElementChild.querySelectorAll(".dlf-operator");
    let isNegated = false;
    let operator = "and";
    operatorElems.forEach((operatorElem) => {
      const value = operatorElem.getAttribute("data-value");
      switch (value) {
        case "not":
          isNegated = true;
          break;
        case "and":
        case "or":
          operator = value;
          break;
      }
    });
    const groupBodyConditionsElem = groupElem.querySelector(".dlf-group-body-conditions");
    const children = Array.from(groupBodyConditionsElem.children).filter(
      (child) => child.classList.contains("dlf-condition") || child.classList.contains("dlf-group")
    ).map((child) => {
      if (child.classList.contains("dlf-condition")) {
        return this.parseCondition(child);
      } else {
        return this.parseGroup(child);
      }
    });
    const expression = [operator, ...children];
    return isNegated ? ["not", expression] : expression;
  }
  parseCondition(conditionElement) {
    const isNegated = conditionElement.querySelector(".dlf-not-operator") ? true : false;
    const inputContainerElem = conditionElement.querySelector(".dlf-condition-input-container");
    const fieldSelectorDivElem = inputContainerElem.firstElementChild;
    const lookupSelectorDivElem = inputContainerElem.children[1];
    const valueInputDivElem = inputContainerElem.lastElementChild;
    const rawField = fieldSelectorDivElem.getAttribute("data-value");
    if (!rawField) {
      throw new Error("Field selector has not 'data-value` attribute!");
    }
    const fieldStr = rawField.replace(",", "__");
    const lookupSelectorElem = lookupSelectorDivElem.firstElementChild;
    if (!lookupSelectorElem) {
      throw new Error("Misses lookupSelectorElem!");
    }
    const lookupStr = lookupSelectorElem.value;
    const valueInputElem = valueInputDivElem.firstElementChild;
    if (!valueInputElem) {
      throw new Error("Misses valueInputElem!");
    }
    const valueType = valueInputElem.type;
    let value = valueInputElem.value;
    switch (valueType) {
      case "datetime-local":
        value = new Date(value).toISOString();
        break;
      case "number":
        value = Number(value);
        break;
      case "checkbox":
        value = value == "on" ? true : false;
        break;
    }
    const condition = [
      "=",
      `${fieldStr}__${lookupStr}`,
      value
    ];
    return isNegated ? ["not", condition] : condition;
  }
};

exports.FilterEditor = FilterEditor;
exports.getValueInput = getValueInput;
exports.isLispyAndOrExpr = isLispyAndOrExpr;
exports.isLispyConditionExpr = isLispyConditionExpr;
exports.isLispyExpression = isLispyExpression;
exports.isLispyNotExpr = isLispyNotExpr;
exports.isLispyOperator = isLispyOperator;
exports.isSchemaField = isSchemaField;
//# sourceMappingURL=lib.js.map
//# sourceMappingURL=lib.js.map